#include "swo_util.h"
#include "stm32h7xx.h" // For ITM registers, __NOP() and ITM_TCR_ITMENA_Msk, ITM_TER_TER_Msk
#include <stddef.h>    // For NULL
#include <stdint.h>    // For uint32_t

// Core function to send a character to a specific SWO ITM port
// Adapted from ITM_SendChar() in the CMSIS-Core
void swo_util_putc(char c, unsigned int channel) {
    if (channel >= 32) return; // Invalid channel

    // Check if ITM is enabled and the specific channel is enabled
    if (((ITM->TCR & ITM_TCR_ITMENA_Msk) != 0UL) &&  /* ITM enabled */
        ((ITM->TER & (1UL << channel)) != 0UL))      /* ITM Port enabled */
    {
        while (ITM->PORT[channel].u32 == 0UL) {
            __NOP(); // Wait until STIM Port is ready for new data
        }
        ITM->PORT[channel].u8 = (uint8_t)c;
    }
}

// Core function to send a null-terminated string to a specific SWO ITM port
void swo_util_puts(const char *s, unsigned int channel) {
    if (channel >= 32 || s == NULL) return; // Invalid channel or null string

    // Check if ITM is enabled and the specific channel is enabled
    if (((ITM->TCR & ITM_TCR_ITMENA_Msk) != 0UL) &&  /* ITM enabled */
        ((ITM->TER & (1UL << channel)) != 0UL))      /* ITM Port enabled */
    {
        while (*s) {
            while (ITM->PORT[channel].u32 == 0UL) {
                __NOP(); // Wait until STIM Port is ready for new data
            }
            ITM->PORT[channel].u8 = (uint8_t)(*s++);
        }
    }
}

// Checks if a specific ITM channel is active and ready for output
bool swo_util_is_channel_active(unsigned int channel) {
    if (channel >= 32) return false;
    bool itm_enabled = ((ITM->TCR & ITM_TCR_ITMENA_Msk) != 0UL);
    bool channel_enabled_raw = ((ITM->TER & (1UL << channel)) != 0UL);
    return itm_enabled && channel_enabled_raw;
}
